@@ -27,7 +27,10 @@ def f_contiguous_strides(itemsize, shape):
     if shape:
         strides = [itemsize]
         for s in shape[:-1]:
-            strides.append(strides[-1]*s)
+            # NOTE: max(1, s) is used to handle 0-sized axes in `shape`;
+            # the stride for `shape[i] <= 1` doesn't matter, but letting it be 0
+            # is not a good idea: https://github.com/inducer/arraycontext/pull/91
+            strides.append(strides[-1]*max(1, s))
         return tuple(strides)
     else:
         return ()
@@ -37,13 +40,19 @@ def c_contiguous_strides(itemsize, shape):
     if shape:
         strides = [itemsize]
         for s in shape[:0:-1]:
-            strides.append(strides[-1]*s)
+            # NOTE: max(1, s) is used to handle 0-sized axes in `shape`;
+            # the stride for `shape[i] <= 1` doesn't matter, but letting it be 0
+            # is not a good idea: https://github.com/inducer/arraycontext/pull/91
+            strides.append(strides[-1]*max(1, s))
         return tuple(strides[::-1])
     else:
         return ()
 
 
 def equal_strides(strides1, strides2, shape):
+    if strides1 == strides2:
+        return True
+
     if len(strides1) != len(strides2) or len(strides2) != len(shape):
         return False
 
@@ -55,11 +64,16 @@ def equal_strides(strides1, strides2, shape):
 
 
 def is_f_contiguous_strides(strides, itemsize, shape):
-    return equal_strides(strides, f_contiguous_strides(itemsize, shape), shape)
+    from pytools import product
+    return (
+            equal_strides(strides, f_contiguous_strides(itemsize, shape), shape)
+            or product(shape) == 0)  # noqa: W503
 
 
 def is_c_contiguous_strides(strides, itemsize, shape):
-    return equal_strides(strides, c_contiguous_strides(itemsize, shape), shape)
+    from pytools import product
+    return (equal_strides(strides, c_contiguous_strides(itemsize, shape), shape)
+            or product(shape) == 0)  # noqa: W503
 
 
 class ArrayFlags:
@@ -70,6 +84,15 @@ class ArrayFlags:
             ary.strides, ary.dtype.itemsize, ary.shape)
         self.forc = self.f_contiguous or self.c_contiguous
 
+    def __repr__(self):
+        return (
+                f"  C_CONTIGUOUS : {self.c_contiguous}\n"
+                f"  F_CONTIGUOUS : {self.f_contiguous}"
+                )
+
+    def __str__(self):
+        return repr(self)
+
 
 def get_common_dtype(obj1, obj2, allow_double):
     # Yes, numpy behaves differently depending on whether
@@ -131,7 +154,7 @@ try:
         raise RuntimeError("numpy's as_strided is broken")
 
     as_strided = _as_strided
-except:
+except Exception:
     # stolen from numpy to be compatible with older versions of numpy
     class _DummyArray:
         """ Dummy object that just exists to hang __array_interface__ dictionaries
@@ -153,8 +176,8 @@ except:
         # This fixes a lot of errors on pypy since DummyArray hack does not
         # currently (2014/May/17) on pypy.
 
-        if ((shape is None or x.shape == shape) and
-            (strides is None or x.strides == strides)):
+        if ((shape is None or x.shape == shape)
+                and (strides is None or x.strides == strides)):  # noqa: W503
             return x
         if not x.dtype.isbuiltin:
             if shape is None:
@@ -185,9 +208,9 @@ except:
 
         interface = dict(x.__array_interface__)
         if shape is not None:
-            interface['shape'] = tuple(shape)
+            interface["shape"] = tuple(shape)
         if strides is not None:
-            interface['strides'] = tuple(strides)
+            interface["strides"] = tuple(strides)
         return np.asarray(_DummyArray(interface, base=x))
 
 # }}}
