@@ -118,7 +118,7 @@ namespace
   py::tuple mem_alloc_pitch_wrap(
       size_t width, size_t height, unsigned int access_size)
   {
-    std::auto_ptr<device_allocation> da;
+    std::unique_ptr<device_allocation> da;
     Py_ssize_t pitch = mem_alloc_pitch(
         da, width, height, access_size);
     return py::make_tuple(
@@ -571,7 +571,7 @@ namespace
           py::stl_input_iterator<npy_intp>(),
           back_inserter(dims));
 
-    std::auto_ptr<Allocation> alloc(
+    std::unique_ptr<Allocation> alloc(
         new Allocation(
           tp_descr->elsize*pycuda::size_from_dims(dims.size(), &dims.front()),
           par1)
@@ -612,7 +612,7 @@ namespace
       throw pycuda::error("register_host_memory", CUDA_ERROR_INVALID_VALUE,
           "ary argument is not contiguous");
 
-    std::auto_ptr<registered_host_memory> regmem(
+    std::unique_ptr<registered_host_memory> regmem(
         new registered_host_memory(
           PyArray_DATA(ary.ptr()), PyArray_NBYTES(ary.ptr()), flags, ary));
 
@@ -897,6 +897,36 @@ BOOST_PYTHON_MODULE(_driver)
     .value("MANAGED_MEMORY", CU_DEVICE_ATTRIBUTE_MANAGED_MEMORY)
     .value("MULTI_GPU_BOARD", CU_DEVICE_ATTRIBUTE_MULTI_GPU_BOARD)
     .value("MULTI_GPU_BOARD_GROUP_ID", CU_DEVICE_ATTRIBUTE_MULTI_GPU_BOARD_GROUP_ID)
+#endif
+#if CUDAPP_CUDA_VERSION >= 8000
+    .value("HOST_NATIVE_ATOMIC_SUPPORTED", CU_DEVICE_ATTRIBUTE_HOST_NATIVE_ATOMIC_SUPPORTED)
+    .value("SINGLE_TO_DOUBLE_PRECISION_PERF_RATIO", CU_DEVICE_ATTRIBUTE_SINGLE_TO_DOUBLE_PRECISION_PERF_RATIO)
+    .value("PAGEABLE_MEMORY_ACCESS", CU_DEVICE_ATTRIBUTE_PAGEABLE_MEMORY_ACCESS)
+    .value("CONCURRENT_MANAGED_ACCESS", CU_DEVICE_ATTRIBUTE_CONCURRENT_MANAGED_ACCESS)
+    .value("COMPUTE_PREEMPTION_SUPPORTED", CU_DEVICE_ATTRIBUTE_COMPUTE_PREEMPTION_SUPPORTED)
+    .value("CAN_USE_HOST_POINTER_FOR_REGISTERED_MEM", CU_DEVICE_ATTRIBUTE_CAN_USE_HOST_POINTER_FOR_REGISTERED_MEM)
+#endif
+#if CUDAPP_CUDA_VERSION >= 9000
+    .value("MAX_SHARED_MEMORY_PER_BLOCK_OPTIN", CU_DEVICE_ATTRIBUTE_MAX_SHARED_MEMORY_PER_BLOCK_OPTIN)
+#endif
+#if CUDAPP_CUDA_VERSION >= 9020
+    .value("PAGEABLE_MEMORY_ACCESS_USES_HOST_PAGE_TABLES", CU_DEVICE_ATTRIBUTE_PAGEABLE_MEMORY_ACCESS_USES_HOST_PAGE_TABLES)
+    .value("DIRECT_MANAGED_MEM_ACCESS_FROM_HOST", CU_DEVICE_ATTRIBUTE_DIRECT_MANAGED_MEM_ACCESS_FROM_HOST)
+#endif
+#if CUDAPP_CUDA_VERSION >= 10020
+    .value("HANDLE_TYPE_POSIX_FILE_DESCRIPTOR_SUPPORTED", CU_DEVICE_ATTRIBUTE_HANDLE_TYPE_POSIX_FILE_DESCRIPTOR_SUPPORTED)
+    .value("HANDLE_TYPE_WIN32_HANDLE_SUPPORTED", CU_DEVICE_ATTRIBUTE_HANDLE_TYPE_WIN32_HANDLE_SUPPORTED)
+    .value("HANDLE_TYPE_WIN32_KMT_HANDLE_SUPPORTED", CU_DEVICE_ATTRIBUTE_HANDLE_TYPE_WIN32_KMT_HANDLE_SUPPORTED)
+#endif
+#if CUDAPP_CUDA_VERSION >= 11000
+    .value("MAX_PERSISTING_L2_CACHE_SIZE", CU_DEVICE_ATTRIBUTE_MAX_PERSISTING_L2_CACHE_SIZE)
+    .value("MAX_BLOCKS_PER_MULTIPROCESSOR", CU_DEVICE_ATTRIBUTE_MAX_BLOCKS_PER_MULTIPROCESSOR)
+    .value("GENERIC_COMPRESSION_SUPPORTED", CU_DEVICE_ATTRIBUTE_GENERIC_COMPRESSION_SUPPORTED)
+    .value("RESERVED_SHARED_MEMORY_PER_BLOCK", CU_DEVICE_ATTRIBUTE_RESERVED_SHARED_MEMORY_PER_BLOCK)
+#endif
+#if CUDAPP_CUDA_VERSION >= 11020
+    .value("READ_ONLY_HOST_REGISTER_SUPPORTED", CU_DEVICE_ATTRIBUTE_READ_ONLY_HOST_REGISTER_SUPPORTED)
+    .value("MEMORY_POOLS_SUPPORTED", CU_DEVICE_ATTRIBUTE_MEMORY_POOLS_SUPPORTED)
 #endif
     ;
 #if CUDAPP_CUDA_VERSION >= 4000
@@ -944,6 +974,13 @@ BOOST_PYTHON_MODULE(_driver)
 #if CUDAPP_CUDA_VERSION >= 3000 && defined(CUDAPP_POST_30_BETA)
     .value("PTX_VERSION", CU_FUNC_ATTRIBUTE_PTX_VERSION)
     .value("BINARY_VERSION", CU_FUNC_ATTRIBUTE_BINARY_VERSION)
+#endif
+#if CUDAPP_CUDA_VERSION >= 10000 /* guessed */
+    .value("CACHE_MODE_CA", CU_FUNC_ATTRIBUTE_CACHE_MODE_CA)
+    .value("MAX_DYNAMIC_SHARED_SIZE_BYTES",
+        CU_FUNC_ATTRIBUTE_MAX_DYNAMIC_SHARED_SIZE_BYTES)
+    .value("PREFERRED_SHARED_MEMORY_CARVEOUT",
+        CU_FUNC_ATTRIBUTE_PREFERRED_SHARED_MEMORY_CARVEOUT)
 #endif
     .value("MAX", CU_FUNC_ATTRIBUTE_MAX)
     ;
@@ -1251,6 +1288,9 @@ BOOST_PYTHON_MODULE(_driver)
 #if CUDAPP_CUDA_VERSION >= 2020
       .DEF_SIMPLE_METHOD(get_attribute)
 #endif
+#if CUDAPP_CUDA_VERSION >= 10000
+      .DEF_SIMPLE_METHOD(set_attribute)
+#endif
 #if CUDAPP_CUDA_VERSION >= 3000 && defined(CUDAPP_POST_30_BETA)
       .DEF_SIMPLE_METHOD(set_cache_config)
 #endif
