@@ -17,16 +17,18 @@ class TestGPUArray:
     def test_pow_array(self):
         a = np.array([1, 2, 3, 4, 5]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
+        b = np.array([1, 2, 3, 4, 5]).astype(np.float64)
+        b_gpu = gpuarray.to_gpu(b)
 
-        result = pow(a_gpu, a_gpu).get()
-        assert (np.abs(a ** a - result) < 1e-3).all()
+        result = pow(a_gpu, b_gpu).get()
+        np.testing.assert_allclose(a ** b, result, rtol=1e-6)
 
-        result = (a_gpu ** a_gpu).get()
-        assert (np.abs(pow(a, a) - result) < 1e-3).all()
+        result = (a_gpu ** b_gpu).get()
+        np.testing.assert_allclose(pow(a, b), result, rtol=1e-6)
 
-        a_gpu **= a_gpu
+        a_gpu **= b_gpu
         a_gpu = a_gpu.get()
-        assert (np.abs(pow(a, a) - a_gpu) < 1e-3).all()
+        np.testing.assert_allclose(pow(a, b), a_gpu, rtol=1e-6)
 
     @mark_cuda_test
     def test_pow_number(self):
@@ -34,11 +36,26 @@ class TestGPUArray:
         a_gpu = gpuarray.to_gpu(a)
 
         result = pow(a_gpu, 2).get()
-        assert (np.abs(a ** 2 - result) < 1e-3).all()
+        np.testing.assert_allclose(a ** 2, result, rtol=1e-6)
 
         a_gpu **= 2
         a_gpu = a_gpu.get()
-        assert (np.abs(a ** 2 - a_gpu) < 1e-3).all()
+        np.testing.assert_allclose(a ** 2, a_gpu, rtol=1e-6)
+
+    @mark_cuda_test
+    def test_rpow_array(self):
+        scalar = np.random.rand()
+        a = abs(np.random.rand(10))
+        a_gpu = gpuarray.to_gpu(a)
+
+        result = (scalar ** a_gpu).get()
+        np.testing.assert_allclose(scalar ** a, result)
+
+        result = (a_gpu ** a_gpu).get()
+        np.testing.assert_allclose(a ** a, result)
+
+        result = (a_gpu ** scalar).get()
+        np.testing.assert_allclose(a ** scalar, result)
 
     @mark_cuda_test
     def test_numpy_integer_shape(self):
@@ -154,7 +171,7 @@ class TestGPUArray:
 
     @mark_cuda_test
     def test_substract_array(self):
-        """Test the substraction of two arrays."""
+        """Test the subtraction of two arrays."""
         # test data
         a = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]).astype(np.float32)
         b = np.array([10, 20, 30, 40, 50, 60, 70, 80, 90, 100]).astype(np.float32)
@@ -170,7 +187,7 @@ class TestGPUArray:
 
     @mark_cuda_test
     def test_substract_scalar(self):
-        """Test the substraction of an array and a scalar."""
+        """Test the subtraction of an array and a scalar."""
 
         # test data
         a = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]).astype(np.float32)
@@ -468,6 +485,53 @@ class TestGPUArray:
         a = gpuarray.arange(12, dtype=np.float32)
         assert (np.arange(12, dtype=np.float32) == a.get()).all()
 
+    @mark_cuda_test
+    def test_stack(self):
+
+        orders = ["F", "C"]
+        input_dims_lst = [0, 1, 2]
+
+        for order in orders:
+            for input_dims in input_dims_lst:
+                shape = (2, 2, 2)[:input_dims]
+                axis = -1 if order == "F" else 0
+
+                from numpy.random import default_rng
+                rng = default_rng()
+                x_in = rng.random(size=shape)
+                y_in = rng.random(size=shape)
+                x_in = x_in if order == "C" else np.asfortranarray(x_in)
+                y_in = y_in if order == "C" else np.asfortranarray(y_in)
+
+                x_gpu = gpuarray.to_gpu(x_in)
+                y_gpu = gpuarray.to_gpu(y_in)
+
+                numpy_stack = np.stack((x_in, y_in), axis=axis)
+                gpuarray_stack = gpuarray.stack((x_gpu, y_gpu), axis=axis)
+
+                np.testing.assert_allclose(gpuarray_stack.get(), numpy_stack)
+
+                assert gpuarray_stack.shape == numpy_stack.shape
+
+    @mark_cuda_test
+    def test_concatenate(self):
+
+        from pycuda.curandom import rand as curand
+
+        a_dev = curand((5, 15, 20), dtype=np.float32)
+        b_dev = curand((4, 15, 20), dtype=np.float32)
+        c_dev = curand((3, 15, 20), dtype=np.float32)
+        a = a_dev.get()
+        b = b_dev.get()
+        c = c_dev.get()
+
+        cat_dev = gpuarray.concatenate((a_dev, b_dev, c_dev))
+        cat = np.concatenate((a, b, c))
+
+        np.testing.assert_allclose(cat, cat_dev.get())
+
+        assert cat.shape == cat_dev.shape
+
     @mark_cuda_test
     def test_reverse(self):
         a = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]).astype(np.float32)
